@@ -13,6 +13,19 @@
 // clsync should be used, if there's more than 5-10 nodes. So the limit in 255 is quite enough. :)
 #define MAXNODES			((1<<8)-1)
 
+// max user/group lengths
+#define USER_LEN			(1<<8)
+#define GROUP_LEN			USER_LEN
+
+// control socket listen backlog (man 2 listen)
+#define SOCKET_BACKLOG			2
+
+// control socket connections limit in clsync
+#define SOCKET_MAX_CLSYNC		8
+
+// control socket connections limit in libclsync
+#define SOCKET_MAX_LIBCLSYNC		(1<<16)
+
 // children count limit
 #define MAXCHILDREN			(1<<8)
 
@@ -33,6 +46,8 @@
 #define DEFAULT_CLUSTERSDLMAX		32
 #define DEFAULT_CONFIG_BLOCK		"default"
 #define DEFAULT_RETRIES			1
+#define DEFAULT_VERBOSE			3
+#define DEFAULT_DUMPDIR			"/tmp/clsync-dump-%label%"
 
 #define FANOTIFY_FLAGS			(FAN_CLOEXEC|FAN_UNLIMITED_QUEUE|FAN_UNLIMITED_MARKS)
 #define FANOTIFY_EVFLAGS		(O_LARGEFILE|O_RDONLY|O_CLOEXEC)
@@ -57,3 +72,6 @@
 #define CONFIG_PATHS 			{ ".clsync.conf", "/etc/clsync/clsync.conf", NULL } /* "~/.clsync.conf" and "/etc/clsync/clsync.conf" */
 
 #define API_PREFIX			"clsyncapi_"
+
+#define DUMP_DIRMODE			0750
+#define DUMP_FILEMODE			0644
