@@ -17,6 +17,9 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#ifndef __CLSYNC_INDEXES_H
+#define __CLSYNC_INDEXES_H
+
 #include <glib.h>
 
 struct indexes {
@@ -27,6 +30,9 @@ struct indexes {
 	GHashTable *exc_fpath_coll_ht[QUEUE_MAX];	// excluded file path aggregation hashtable for every queue
 	GHashTable *fpath2ei_coll_ht[QUEUE_MAX];	// "file path -> event information" aggregation hashtable for every queue
 	GHashTable *out_lines_aggr_ht;			// output lines aggregation hashtable
+	GHashTable *nonthreaded_syncing_fpath2ei_ht;	// events that are synchronized in signle-mode (non threaded)
 };
 typedef struct indexes indexes_t;
 
+#endif
+
