@@ -17,6 +17,8 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-extern int sync_run(struct options *options);
+extern int sync_run(struct ctx *ctx);
+extern int sync_dump(struct ctx *ctx, const char *const dest_dir);
 extern int sync_term(int exitcode);
+extern int threads_foreach(int (*funct)(threadinfo_t *, void *), state_t state, void *arg);
 
